@@ -12,30 +12,53 @@
 #
 ##############################################################################
 
-import ZODB.config
 import ZODB.FileStorage
+import ZODB.config
+import ZODB.serialize
 import logging
 import optparse
 import pkg_resources
 import pprint
 import sys
+import time
 import zodbupdate.update
+import zodbupdate.utils
 
 
 parser = optparse.OptionParser(
     description="Updates all references to classes to their canonical location.")
-parser.add_option("-f", "--file",
-                  help="load FileStorage")
-parser.add_option("-c", "--config",
-                  help="load storage from config file")
-parser.add_option("-n", "--dry-run", action="store_true",
-                  help="perform a trial run with no changes made")
-parser.add_option("-s", "--save-renames",
-                  help="save automatically determined rename rules to file")
-parser.add_option("-q", "--quiet", action="store_true",
-                  help="suppress non-error messages")
-parser.add_option("-v", "--verbose", action="store_true",
-                  help="more verbose output")
+parser.add_option(
+    "-f", "--file",
+    help="load FileStorage")
+parser.add_option(
+    "-c", "--config",
+    help="load storage from config file")
+parser.add_option(
+    "-n", "--dry-run", action="store_true",
+    help="perform a trial run with no changes made")
+parser.add_option(
+    "-s", "--save-renames",
+    help="save automatically determined rename rules to file")
+parser.add_option(
+    "-q", "--quiet", action="store_true",
+    help="suppress non-error messages")
+parser.add_option(
+    "-v", "--verbose", action="store_true",
+    help="more verbose output")
+parser.add_option(
+    "-o", "--oid",
+    help="start with provided oid in hex format, ex: 0xaa1203")
+parser.add_option(
+    "-d", "--debug", action="store_true",
+    help="post mortem pdb on failure")
+parser.add_option(
+    "-p", "--pickler", default="C",
+    help="chooser unpickler implementation C or Python (default C)")
+parser.add_option(
+    "--pack", action="store_true", dest="pack",
+    help="pack the storage when done. use in conjunction of -c if you have blobs storage")
+
+
 
 class DuplicateFilter(object):
 
@@ -54,6 +77,10 @@ duplicate_filter = DuplicateFilter()
 def main():
     options, args = parser.parse_args()
 
+    if options.pickler not in zodbupdate.utils.UNPICKLERS:
+        raise SystemExit(u'Invalid pickler chosen. Available picklers: %s' %
+                         ', '.join(zodbupdate.utils.UNPICKLERS))
+
     if options.quiet:
         level = logging.ERROR
     elif options.verbose:
@@ -77,6 +104,10 @@ def main():
         raise SystemExit(
             u'Exactly one of --file or --config must be given.')
 
+    start_at = '0x00'
+    if options.oid:
+        start_at = options.oid
+
     rename_rules = {}
     for entry_point in pkg_resources.iter_entry_points('zodbupdate'):
         rules = entry_point.load()
@@ -85,8 +116,12 @@ def main():
                       (len(rules), entry_point.module_name, entry_point.name))
 
     updater = zodbupdate.update.Updater(
-        storage, dry=options.dry_run,
-        renames=rename_rules)
+        storage,
+        dry=options.dry_run,
+        renames=rename_rules,
+        start_at=start_at,
+        debug=options.debug,
+        pickler_name=options.pickler)
 
     try:
         updater()
@@ -105,5 +140,8 @@ def main():
         f = open(options.save_renames, 'w')
         f.write('renames = %s' % pprint.pformat(rename_rules))
         f.close()
+    if options.pack:
+        print 'Packing storage ...'
+        storage.pack(time.time(), ZODB.serialize.referencesf)
     storage.close()
 
